#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

ushort *gflog, *gfilog;
uint w;

#define NW (1 << w)

int setup_tables(uint prim_poly)
{
	uint x_to_w, b, log;
	x_to_w = 1 << w;
	
	gflog = (ushort *) malloc(sizeof(ushort) * x_to_w);
	gfilog = (ushort *) malloc(sizeof(ushort) * x_to_w);

	b = 1;
	for (log = 0; log < x_to_w-1; log++) {
		gflog[b] = (ushort) log;
		gfilog[log] = (ushort) b;
		b = b << 1;
		if (b & x_to_w) b = b ^ prim_poly;
	}
	return 0;
}

void print_tables() {
	uint x_to_w, i;
	x_to_w = 1 << w;

	printf("Log table:\n");
	for (i = 0; i < x_to_w; i++) {
		printf("%6d %6d\n", i, gflog[i]);
	}

	printf("\nInverse log table:\n");
	for (i = 0; i < x_to_w; i++) {
		printf("%6d %6d\n", i, gfilog[i]);
	}
}


int gfmult(int a, int b)
{
	int sum_log;

	if (a == 0 || b == 0)
		return 0;
	sum_log = gflog[a] + gflog[b];
	if (sum_log >= NW-1)
		sum_log -= NW-1;
	return gfilog[sum_log];
}

int gfdiv(int a, int b)
{
	int diff_log;

	if (a == 0)
		return 0;
	if (b == 0)
		return -1;
	diff_log = gflog[a] - gflog[b];
	if (diff_log < 0)
		diff_log += NW-1;
	return gfilog[diff_log];
}


void print_gfmult(int a, int b)
{
	printf("%d*%d = %d\n", a, b, gfmult(a, b));
}

void print_gfdiv(int a, int b)
{
	printf("%d/%d = %d\n", a, b, gfdiv(a, b));
}

void compute_syndromes()
{
	int i, j;
	unsigned short s[4], p[4];
	unsigned char buf[1024];
	memset(s, 0, sizeof(s));
	memset(buf, 0, sizeof(buf));
	if (read(0, buf, 512) != 512) {
		printf("Short read!\n");
		return;
	}
	for (j = 1; j < 5; j++) {
		for (i = 0; i < 103; i+=5) {
			p[0] = ((buf[i+0] & 0777)>>0) | ((buf[i+1] & 0003)<<8);
			p[1] = ((buf[i+1] & 0774)>>2) | ((buf[i+2] & 0017)<<6);
			p[2] = ((buf[i+2] & 0360)>>4) | ((buf[i+3] & 0077)<<4);
			p[3] = ((buf[i+3] & 0300)>>6) | ((buf[i+4] & 0377)<<2);
			s[j-1] ^= gfmult(p[0], gfilog[(i*j) % (w-1)]);
			s[j-1] ^= gfmult(p[1], gfilog[(i*j) % (w-1)]);
			s[j-1] ^= gfmult(p[2], gfilog[(i*j) % (w-1)]);
			s[j-1] ^= gfmult(p[3], gfilog[(i*j) % (w-1)]);
		}
		printf("s[%d] = %d\n", j, s[j-1]);
	}
}

int main(int argc, char *argv[]) {
	uint prim_poly;

	if (argc < 3) {
		printf("Usage: %s <w> <prim_poly>\n", argv[0]);
		return 1;
	}

	w = atoi(argv[1]);
	prim_poly = atoi(argv[2]);

	setup_tables(prim_poly);
	/*
	print_tables(w);
	print_gfmult(3, 7, w);
	print_gfmult(13, 10, w);
	print_gfdiv(13, 10, w);
	print_gfdiv(3, 7, w);
	*/

	compute_syndromes();
	return 0;
}
